@@ -52,6 +52,7 @@ single-row queries in the future.
             },
             author => { insert => undef  },
         },
+        "Doing that stuff runs correct SQL", # optional
     );
 
 
@@ -71,6 +72,15 @@ single-row queries in the future.
         author => { insert => undef },
     });
 
+    # or, with test description
+    $queries->test(
+        {
+            book   => { select => "<= 2"},
+            author => { insert => undef },
+        },
+        "Doing that stuff runs correct SQL", # optional
+    );
+
 
 
 =head1 USAGE
@@ -156,7 +166,7 @@ Many DBIC methods are context sensitive, and in scalar context might
 just return an unrealized resultset rather than execute a query and
 return the resulting rows. If you're unsure, assigning the query to an
 array will make it run in list context and therefore execute the SQL
-query.
+query. Or you can call C<-&gt;>all> on the resultset object.
 
 
 =head2 DBIC_TRACE
@@ -164,18 +174,19 @@ query.
 Normally, setting the ENV variable DBIC_TRACE can be used to "warn"
 the DBIC queries.
 
-Test::DBIC:ExpectedQueries uses the same mechanism as DBIC_TRACE, so
-while the code is run under the test the normal DBIC_TRACE will not
+Test::DBIC:ExpectedQueries uses the same mechanism as DBIC_TRACE does,
+so while the code is run under the test the normal DBIC_TRACE will not
 happen.
 
 
 
 =head1 SUBROUTINES
 
-=head2 expected_queries( $schema, $sub_ref, $expected_table_operations = {} ) : $result | @result
+=head2 expected_queries( $schema, $sub_ref, $expected_table_operations = {}, $description? ) : $result | @result
 
 Run $sub_ref and collect stats for queries executed on $schema, then
-test that they match the $expected_table_operations.
+test (using $description) that they match the
+$expected_table_operations.
 
 Return the return value of $sub_ref->().
 
@@ -238,15 +249,16 @@ You can call $queries->run() multiple times to add to the collected
 stats before finally calling $queries->test().
 
 
-=head2 test( $expected_table_operations = {} ) : $is_passing
+=head2 test( $expected_table_operations = {}, $description? ) : $is_passing
 
-Test the collected queries against $expected_table_operations (see
-above) and either pass or fail a Test::More test.
+Test (using $description) the collected queries against
+$expected_table_operations (see above) and either pass or fail a
+Test::More test.
 
-If the test fails, list all queries relating to the tables with
+If the test fails, C<diag> all queries relating to the tables with
 unexpected activity.
 
-If anything failed to be identified as a known query, always list
+If anything failed to be identified as a known query, always C<note>
 those queries. But don't fail the test just because of it.
 
 Reset the collected stats, so subsequent calls to ->run() start with a
@@ -330,7 +342,7 @@ queries until after they have been run.
 =cut
 
 package Test::DBIC::ExpectedQueries;
-$Test::DBIC::ExpectedQueries::VERSION = '2.000';
+$Test::DBIC::ExpectedQueries::VERSION = '2.001';
 use Moo;
 use Exporter::Tiny;
 BEGIN {extends "Exporter::Tiny"};
@@ -342,6 +354,8 @@ use Try::Tiny;
 use Carp;
 use DBIx::Class;
 use Devel::StackTrace;
+use autobox::Core;
+use autobox::Transform;
 
 use Test::DBIC::ExpectedQueries::Query;
 
@@ -350,7 +364,7 @@ use Test::DBIC::ExpectedQueries::Query;
 ### Simple procedural interface
 
 sub expected_queries {
-    my ($schema, $subref, $expected) = @_;
+    my ($schema, $subref, $expected, $description) = @_;
     $expected ||= {};
     local $Test::Builder::Level = $Test::Builder::Level + 1;
 
@@ -364,7 +378,7 @@ sub expected_queries {
         $return_values = [ scalar $queries->run($subref) ];
     }
 
-    $queries->test($expected);
+    $queries->test($expected, $description);
 
     return @$return_values if wantarray();
     return $return_values->[0];
@@ -407,25 +421,30 @@ sub _build_ignore_classes {
     my $self = shift;
     return [
         # "main",
-        "Test::DBIC::ExpectedQueries",
         "Class::MOP::Method::Wrapped",
-        "Moose::Meta::Method::Delegation",
         "Context::Preserve",
-        # "DBIx::Class",
-        # "DBIx::Class::Schema",
-        # "DBIx::Class::Storage::BlockRunner",
+        "DBIx::Class",
         "DBIx::Class::ResultSet",
         "DBIx::Class::Row",
+        "DBIx::Class::Row",
+        "DBIx::Class::Schema",
+        "DBIx::Class::Storage::BlockRunner",
         "DBIx::Class::Storage::DBI",
         "DBIx::Class::Storage::Statistics",
-        "DBIx::Class::Row",
+        "Mojo::IOLoop",
+        "Mojo::Promise",
+        "Mojo::Reactor",
+        "Moose::Meta::Method::Delegation",
+        "Test::Builder",
         "Test::Builder",
         "Test::Class",
         "Test::Class::Moose",
-        "Test::Class::Moose::Runner",
+        "Test::Class::Moose::Executor::Sequential",
         "Test::Class::Moose::Report::Method",
         "Test::Class::Moose::Role::Executor",
-        "Test::Class::Moose::Executor::Sequential",
+        "Test::Class::Moose::Runner",
+        "Test::DBIC::ExpectedQueries",
+        "Test::More",
         "Try::Tiny",
         "Try::Tiny::Catch",
     ];
@@ -435,14 +454,20 @@ sub _stack_trace {
     my $self = shift;
 
     my $trace = Devel::StackTrace->new(
-        message      => "executed",
+        message      => "SQL executed",
         ignore_class => $self->ignore_classes,
     );
-
     my $callers = $trace->as_string;
-    chomp($callers);
-    $callers =~ s/\n/ <-- /gsm;
-    $callers =~ s/=?(HASH|ARRAY)\(0x\w+\)/<$1>/gsm;
+
+    $callers =~ s/=?(HASH|ARRAY|CODE|GLOB)\(0x\w+\)/<$1>/gsm;
+
+    # Indent all but first line
+    my ($first, @rest) = $callers->split(qr/\n/);
+    my $rest = @rest
+        ->filter
+        ->map(sub { "    $_" })
+        ->join("\n");
+    $callers = "$first\n$rest";
 
     return $callers;
 }
@@ -496,8 +521,9 @@ sub run {
 
 sub test {
     my $self = shift;
-    my ($expected) = @_;
+    my ($expected, $test_description) = @_;
     $expected ||= {};
+    $test_description ||= "Expected queries for tables";
     local $Test::Builder::Level = $Test::Builder::Level + 1;
 
     my $failure_message = $self->check_table_operation_counts($expected);
@@ -506,14 +532,15 @@ sub test {
     $self->clear_queries();
     $self->clear_table_operation_count();
 
-
-    my $test_description = "Expected queries for tables";
     if($failure_message) {
-        fail("$test_description:\n\n$failure_message$unknown_warning");
+        fail($test_description);
+        diag("\n$failure_message");
+        $unknown_warning and note($unknown_warning);
         return 0;
     }
 
-    pass("$test_description$unknown_warning");
+    pass($test_description);
+    $unknown_warning and note($unknown_warning);
     return 1;
 }
 
